!!****if* source/Driver/DriverMain/Unsplit/Driver_evolveFlash
!!
!! NAME
!!
!!  Driver_evolveFlash
!!
!! SYNOPSIS
!!
!!  Driver_evolveFlash()
!!
!! DESCRIPTION
!!
!! This routine implements a directionally unsplit integrator for time 
!! advancement. A single step in this driver accounts for all 
!! multidimensional fluxes and updates conserved variables for the next
!! time step. The routine is a default driver for the Unsplit Staggered
!! Mesh (USM) MHD scheme.
!!
!! NOTES
!!
!! The Driver unit uses a few unit scope variables that are
!! accessible to all routines within the unit, but not to the
!! routines outside the unit. These variables begin with "dr_"
!! like, dr_myPE or dr_dt, dr_beginStep, and are stored in fortran
!! module Driver_data (in file Driver_data.F90. The other variables
!! are local to the specific routine and do not have the prefix "dr_"
!!
!!
!!***


#ifdef DEBUG_ALL
#define DEBUG_DRIVER
#endif

subroutine Driver_evolveFlash()

  use Driver_data,         ONLY : dr_myPE, dr_numProcs, dr_nbegin,       &
                                  dr_nend, dr_dt, dr_wallClockTimeLimit, &
                                  dr_tmax, dr_simTime, dr_redshift,      &
                                  dr_nstep, dr_dtOld, dr_dtNew,          &
                                  dr_restart, dr_elapsedWCTime,          &
                                  dr_redshiftOld, dr_useRedshift,        &
                                  dr_redshiftfinal
  use Driver_interface,    ONLY : Driver_sourceTerms, Driver_computeDt, &
                                  Driver_getElapsedWCTime
  use Logfile_interface,   ONLY : Logfile_stamp, Logfile_close
  use Timers_interface,    ONLY : Timers_start, Timers_stop, &
                                  Timers_getSummary
  use Particles_interface, ONLY : Particles_advance
  use Grid_interface,      ONLY : Grid_getLocalNumBlks, &
                                  Grid_getListOfBlocks, &
                                  Grid_updateRefinement
  use Hydro_interface, ONLY : Hydro
  use Gravity_interface,   ONLY : Gravity_potentialListOfBlocks
  use IO_interface,        ONLY : IO_output,IO_outputFinal
  use Cosmology_interface, ONLY:  Cosmology_redshiftHydro, &
       Cosmology_solveFriedmannEqn, Cosmology_getRedshift

  implicit none

#include "constants.h"
#include "Flash.h"
 include "Flash_mpi.h"

  integer   :: localNumBlocks

  integer :: blockCount
  integer :: blockList(MAXBLOCKS)
  integer :: sweepDummy

  ! for logfile output
  character(len=MAX_STRING_LENGTH), dimension(4,2) :: strBuff
  character(len=15) :: numToStr

  logical :: endRun  

  call Logfile_stamp(dr_myPE, 'Entering evolution loop' , '[Driver_evolveFlash]')
  call Timers_start("evolution")

  do dr_nstep = dr_nBegin, dr_nend
     
     !!Step forward in time. See bottom of loop for time step calculation.
     call Grid_getLocalNumBlks(localNumBlocks)
     call Grid_getListOfBlocks(LEAF,blockList,blockCount)

     if (dr_myPE == MASTER_PE) then

        write (numToStr(1:), '(I10)') dr_nstep
        write (strBuff(1,1), "(A)") "n"
        write (strBuff(1,2), "(A)") trim(adjustl(numToStr))
        
        write (numToStr(1:), "(1PE12.6)") dr_simTime
        write (strBuff(2,1), "(A)") "t"
        write (strBuff(2,2), "(A)") trim(adjustl(numToStr))

        if (.not. dr_useRedshift) then

           write (numToStr(1:), "(1PE12.6)") dr_dt
           write (strBuff(3,1), "(A)") "dt"
           write (strBuff(3,2), "(A)") trim(adjustl(NumToStr))

           call Logfile_stamp(dr_myPE, strBuff(1:3,:), 3, 2, "step")

        else

           write (numToStr(1:), "(F8.3)") dr_redshift
           write (strBuff(3,1), "(A)") "z"
           write (strBuff(3,2), "(A)") trim(adjustl(NumToStr))

           write (numToStr(1:), "(1PE12.6)") dr_dt
           write (strBuff(4,1), "(A)") "dt"
           write (strBuff(4,2), "(A)") trim(adjustl(NumToStr))
           
           call Logfile_stamp(dr_myPE, strBuff, 4, 2, "step")

        endif

     end if


     !--------------------------------------------------------------------
     !- Start Physics Sequence
     !----
#ifdef DEBUG_DRIVER
     print*, 'going into Hydro'  ! DEBUG
#endif

     call Timers_start("cosmology")
     call Cosmology_solveFriedmannEqn(dr_simTime, dr_dt)
     call Timers_stop("cosmology")

     dr_simTime = dr_simTime + dr_dt

     call Timers_start("Hydro")
     call Hydro(dr_myPE, dr_numProcs, blockCount, blockList,   &
                dr_simTime, dr_dt, dr_dtOld,  sweepDummy)
     call Timers_stop("Hydro")

#ifdef DEBUG_DRIVER
  print*, 'return from Hydro timestep'  ! DEBUG
#endif

     call Timers_start("sourceTerms")
     call Driver_sourceTerms(blockCount, blockList, dr_dt)
     call Timers_stop("sourceTerms")
#ifdef DEBUG_DRIVER
     print*,'done source terms'  ! DEBUG
     print*, 'return from Drivers_sourceTerms '  ! DEBUG
#endif
     call Timers_start("Particles_advance")
     call Particles_advance(dr_dtOld, dr_dt)
#ifdef DEBUG_DRIVER
     print*, 'return from Particles_advance '  ! DEBUG
#endif
     call Timers_stop("Particles_advance")
     call Gravity_potentialListOfBlocks(blockCount,blockList)
#ifdef DEBUG_DRIVER
     print*, 'return from Gravity_potential '  ! DEBUG
#endif

     call Timers_start("cosmology")
     call Cosmology_redshiftHydro(dr_myPE, dr_numprocs, blockCount, blockList)
     call Timers_stop("cosmology")

     !----
     !- End Physics Sequence
     !--------------------------------------------------------------------
     call Timers_start("Grid_updateRefinement")
     call Grid_updateRefinement(dr_myPE, dr_nstep, dr_simTime)
     call Timers_stop("Grid_updateRefinement")

     ! backup needed old
     dr_dtOld = dr_dt
#ifdef DEBUG_DRIVER
     print*, 'going into Driver_computeDt '  ! DEBUG
#endif
     ! calculate new
     call Driver_computeDt(dr_myPE,    dr_numProcs,   &
                           dr_nbegin,  dr_nstep,      &
                           dr_simTime, dr_dtOld, dr_dtNew)
#ifdef DEBUG_DRIVER
     print*, 'return from Driver_computeDt '  ! DEBUG
#endif
     ! store new
     dr_dt = dr_dtNew
     
     call Timers_start("io")
     call IO_output(dr_myPE,dr_numProcs,dr_simTime,dr_dt,dr_nstep+1,dr_nbegin,endRun)
     call Timers_stop("io")

     if(endRun) exit

     !! the simulation ends before nend iterations if
     !!  (i)   the simulation time is greater than the maximum time (tmax)
     !!  (ii)  the redshift falls below the minimum redshift  
     !!        (also called zfinal)
     !!  (iii) the wall clock time is greater than the maximum 
     !!        (wall_clock_time_max)

     if (dr_simTime >= dr_tmax) then
        if(dr_myPE == MASTER_PE) print *, "exiting: reached max SimTime"
        exit
     end if

     !!Update redshift from Driver's POV.  Need this for exit condition. -PR   
     !!old redshift needed for accurate restarts.                              
     dr_redshiftOld = dr_redshift
     call Cosmology_getRedshift(dr_redshift)

     call Driver_getElapsedWCTime(dr_elapsedWCTime)
     if (dr_elapsedWCTime >  dr_wallClockTimeLimit) then
        if(dr_myPE == MASTER_PE) print *, "exiting: reached max wall clock time"
        exit
     end if

     if (dr_redshift < dr_redshiftfinal .and. dr_useRedshift) then
        if(dr_mype == MASTER_PE) print *, "exiting: reached redshiftfinal"
        exit
     end if

  enddo

  call Timers_stop("evolution")
  call Logfile_stamp(dr_myPE, 'Exiting evolution loop' , '[Driver_evolveFlash]')
  if(.NOT.endRun) call IO_outputFinal(dr_myPE, dr_numProcs)
  call Timers_getSummary(dr_myPE, dr_nstep)
  call Logfile_stamp(dr_myPE, "FLASH run complete.", "LOGFILE_END")
  call Logfile_close(dr_myPE)

  return
  
end subroutine Driver_evolveFlash



