!!****if* source/flashUtilities/general/Utilities_findMean
!!
!! NAME
!!  Utilities_findMean
!! 
!! SYNOPSIS
!!  Utilities_findMean(integer, intent(in)  :: iSrc,
!!                     integer, intent(in)  :: iType,
!!                     logical, intent(in)  :: bGuardcell
!!                        real, intent(out) :: mean)
!!
!! DESCRIPTION
!!  Calculates the mean of a function
!!
!!
!! ARGUMENTS
!!  iSrc -- the index (e.g. DENS_VAR) into the unk routine to calculate over
!!  iType -- the type of mean.  Valid values will be  (feel free to implement more)
!!     1 = L1 norm = arithmetic average
!!  bGuardcell -- logical indicating whether guard cells should be included in the calculation
!!  mean -- the requested mean
!!
!!
!!***

subroutine Utilities_findMean(iSrc, iType, bGuardcell, mean)
  
  use Driver_interface, ONLY: Driver_abortFlash
  use Grid_interface, ONLY : Grid_getListOfBlocks, &
       Grid_getBlkPhysicalSize, Grid_getBlkIndexLimits, &
       Grid_getBlkPtr, Grid_releaseBlkPtr

  implicit none

#include "constants.h"
#include "Flash.h"
#include "Flash_mpi.h"

  integer, intent(in) :: iSrc, iType
  logical, intent(in) :: bGuardcell
  real, intent(out) :: mean

  real :: localVolume, localSum, blockSum, bvol,  sum
  real :: numZonesInv

  real :: blockVolume, cellVolume, volume
  integer :: blkCount, lb, blockID, i, j, k, ierr
  integer :: ili, iui, jli, jui, kli, kui
  integer, dimension(2,MDIM) :: blkLimits, blkLimitsGC
  integer, dimension(MAXBLOCKS) :: blkList
  real, dimension(MDIM) :: blockSize
  real, dimension(:,:,:,:), pointer :: solnData

!!==============================================================================

  mean = 0.0

  if (iType /= 1) then
     call Driver_abortFlash("[Utilities_findMean] Can only do arithmetic mean with iType=1")
     return
  end if

  localVolume = 0.
  localSum = 0.

  numZonesInv = 1. / real(NXB)
  if (NDIM >= 2) numZonesInv = numZonesInv / real(NYB)
  if (NDIM == 3) numZonesInv = numZonesInv / real(NZB)

  call Grid_getListOfBlocks(LEAF,blkList,blkCount)

  do lb = 1, blkCount

     blockID = blkList(lb)

     call Grid_getBlkPtr(blockID, solnData)
     call Grid_getBlkPhysicalSize(blockID, blockSize)
     call Grid_getBlkIndexLimits(blockID, blkLimits, blkLimitsGC)

     blockVolume = blockSize(IAXIS)
     if (NDIM >= 2) blockVolume = blockVolume * blockSize(JAXIS)
     if (NDIM == 3) blockVolume = blockVolume * blockSize(KAXIS)

     cellVolume = blockVolume * numZonesInv
     localVolume = localVolume + blockVolume
     blockSum = 0.

     if (bGuardcell) then
        ili = blkLimitsGC(LOW,IAXIS)
        iui = blkLImitsGC(HIGH,IAXIS)
        jli = blkLimitsGC(LOW,JAXIS)
        jui = blkLImitsGC(HIGH,JAXIS)
        kli = blkLimitsGC(LOW,KAXIS)
        kui = blkLImitsGC(HIGH,KAXIS)
     else
        ili = blkLimits(LOW,IAXIS)
        iui = blkLImits(HIGH,IAXIS)
        jli = blkLimits(LOW,JAXIS)
        jui = blkLImits(HIGH,JAXIS)
        kli = blkLimits(LOW,KAXIS)
        kui = blkLImits(HIGH,KAXIS)
     endif

     do k = kli, kui
        do j = jli, jui
           do i = ili, iui
              blockSum = blockSum + solnData(iSrc,i,j,k)
           enddo
        enddo
     enddo

     localSum = localSum + blockSum * cellVolume

     call Grid_releaseBlkPtr(blockID, solnData)

  enddo

  call mpi_allreduce ( localSum, sum, 1, FLASH_REAL, & 
       MPI_SUM, MPI_COMM_WORLD, ierr )
  call mpi_allreduce ( localVolume, volume, 1, FLASH_REAL, & 
       MPI_SUM, MPI_COMM_WORLD, ierr )

  mean = sum / volume

  return

end subroutine Utilities_findMean

